/*=============================================================================
Vitalnix User Management Suite
backends/ldap.c - LDAP back-end module
  Copyright © Jan Engelhardt <jengelh [at] gmx de>, 2005 - 2006

  This file is part of Vitalnix 2006 ENTERPRISE EDITION.
  You may not redistribute this file in any form without obtaining
  written permission (includes email) from Alphagate Systems Indp.

  This PDB back-end module comes WITHOUT ANY WARRANTY; without even the
  implied warranty of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
  By using it, you agree to this.
=============================================================================*/
#include <sys/types.h>
#include <errno.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ldap.h>
#include "backends/static-build.h"
#include "libvxpdb/libvxpdb.h"

struct ldap_state {
    // Connection
    LDAP *cn_handle;
    char *cn_host, *cn_socket, *cn_user, *cn_passwd, *cn_database,
         *cn_user2, *cn_passwd2;
    int cn_port;

    // Misc
    long uid_min, uid_max, gid_min, gid_max;
};

// Functions
static int vldap_init(struct vxpdb_state *, const void *);
static int vldap_open(struct vxpdb_state *, long);
static void vldap_close(struct vxpdb_state *);
static void vldap_deinit(struct vxpdb_state *);

static int vldap_useradd(struct vxpdb_state *, const struct vxpdb_user *);
static int vldap_usermod(struct vxpdb_state *, const struct vxpdb_user *, const struct vxpdb_user *);
static int vldap_userdel(struct vxpdb_state *, const struct vxpdb_user *);
static int vldap_userinfo(struct vxpdb_state *, const struct vxpdb_user *, struct vxpdb_user *, size_t);
static void *vldap_usertrav_init(struct vxpdb_state *);
static int vldap_usertrav_walk(struct vxpdb_state *, void *, struct vxpdb_user *);
static void vldap_usertrav_free(struct vxpdb_state *, void *);

static int vldap_groupadd(struct vxpdb_state *, const struct vxpdb_group *);
static int vldap_groupmod(struct vxpdb_state *, const struct vxpdb_group *, const struct vxpdb_group *);
static int vldap_groupdel(struct vxpdb_state *, const struct vxpdb_group *);
static int vldap_groupinfo(struct vxpdb_state *, const struct vxpdb_group *, struct vxpdb_group *, size_t);
static void *vldap_grouptrav_init(struct vxpdb_state *);
static int vldap_grouptrav_walk(struct vxpdb_state *, void *, struct vxpdb_group *);
static void vldap_grouptrav_free(struct vxpdb_state *, void *);

// Variables
COND_STATIC struct vxpdb_module THIS_MODULE = {
    .name           = "LDAP back-end module",
    .author         = "Alphagate Systems <support [at] ahn hopto org>, 2005 - 2006",

    .init           = vldap_init,
    .open           = vldap_open,
    .close          = vldap_close,
    .deinit         = vldap_deinit,

    .useradd        = vldap_useradd,
    .usermod        = vldap_usermod,
    .userdel        = vldap_userdel,
    .userinfo       = vldap_userinfo,
    .usertrav_init  = vldap_usertrav_init,
    .usertrav_walk  = vldap_usertrav_walk,
    .usertrav_free  = vldap_usertrav_free,

    .groupadd       = vldap_groupadd,
    .groupmod       = vldap_groupmod,
    .groupdel       = vldap_groupdel,
    .groupinfo      = vldap_groupinfo,
    .grouptrav_init = vldap_grouptrav_init,
    .grouptrav_walk = vldap_grouptrav_walk,
    .grouptrav_free = vldap_grouptrav_free,
};

//-----------------------------------------------------------------------------
REGISTER_MODULE(ldap, &THIS_MODULE);

static int vldap_init(struct vxpdb_state *this, const void *private_data) {
    struct ldap_state *sp;

    if((sp = this->state = calloc(1, sizeof(struct ldap_state))) == NULL)
        return -errno;

    sp->cn_port = LDAP_PORT;
    return 1;
}

static int vldap_open(struct vxpdb_state *this, long flags) {
    struct ldap_state *sp = this->state;

    if((sp->cn_handle = ldap_init(sp->cn_host, sp->cn_port)) == NULL)
        return -errno;

    ldap_set_option(sp->cn_handle, LDAP_OPT_DEREF, LDAP_DEREF_NEVER);
    ldap_set_option(sp->cn_handle, LDAP_OPT_REFERRALS, LDAP_OPT_OFF);
//    ldap_bind_s(ld, pv->binddn, pv->bindpw, LDAP_AUTH_SIMPLE);

    return 1;
}

static void vldap_close(struct vxpdb_state *this) {
    return;
}

static void vldap_deinit(struct vxpdb_state *this) {       
    struct ldap_state *sp = this->state;
    free(sp);
    return;
}

//-----------------------------------------------------------------------------
static int vldap_useradd(struct vxpdb_state *this, const struct vxpdb_user *u) {
    return 0;
}

static int vldap_usermod(struct vxpdb_state *this, const struct vxpdb_user *srk,
 const struct vxpdb_user *mod)
{
    return 0;
}

static int vldap_userdel(struct vxpdb_state *this, const struct vxpdb_user *u) {
    return 0;
}

static int vldap_userinfo(struct vxpdb_state *this, const struct vxpdb_user *srk,
 struct vxpdb_user *res, size_t size)
{
    return 0;
}

static void *vldap_usertrav_init(struct vxpdb_state *this) {
    return this;
}

static int vldap_usertrav_walk(struct vxpdb_state *this, void *ptr,
 struct vxpdb_user *result)
{
    return 0;
}

static void vldap_usertrav_free(struct vxpdb_state *this, void *ptr) {
    return;
}

//-----------------------------------------------------------------------------
static int vldap_groupadd(struct vxpdb_state *this, const struct vxpdb_group *g) {
    return 0;
}

static int vldap_groupmod(struct vxpdb_state *this, const struct vxpdb_group *srk,
 const struct vxpdb_group *mod)
{
    return 0;
}

static int vldap_groupdel(struct vxpdb_state *this, const struct vxpdb_group *g) {
    return 0;
}

static int vldap_groupinfo(struct vxpdb_state *this, const struct vxpdb_group *srk,
 struct vxpdb_group *res, size_t size)
{
    return 0;
}

static void *vldap_grouptrav_init(struct vxpdb_state *this) {
    return this;
}

static int vldap_grouptrav_walk(struct vxpdb_state *this, void *ptr,
 struct vxpdb_group *result)
{
    return 0;
}

static void vldap_grouptrav_free(struct vxpdb_state *this, void *ptr) {
    return;
}

//=============================================================================
